"""This module provides helpers for writing Sensu plugins"""
from sensu_plugin.plugin import SensuPlugin
from sensu_plugin.check import SensuPluginCheck
from sensu_plugin.metric import SensuPluginMetricJSON
from sensu_plugin.metric import SensuPluginMetricGraphite
from sensu_plugin.metric import SensuPluginMetricStatsd
from sensu_plugin.handler import SensuHandler
import sensu_plugin.pushevent



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 - S. Zachariah Sprackett <zac@sprackett.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

'''
This provides a base SensuHandler class that can be used for writing
python-based Sensu handlers.
'''

from __future__ import print_function
import os
import sys
import json
import requests
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse
from sensu_plugin.utils import get_settings


class SensuHandler(object):
    '''
    Class to be used as a basis for handlers.
    '''
    def __init__(self):
        # Parse the stdin into a global event object
        stdin = sys.stdin.read()
        self.read_event(stdin)

        # Prepare global settings
        self.settings = get_settings()
        self.api_settings = self.get_api_settings()

        # Filter (deprecated) and handle
        self.filter()
        self.handle()

    def read_event(self, check_result):
        '''
        Convert the piped check result (json) into a global 'event' dict
        '''
        try:
            self.event = json.loads(check_result)
            self.event['occurrences'] = self.event.get('occurrences', 1)
            self.event['check'] = self.event.get('check', {})
            self.event['client'] = self.event.get('client', {})
        except Exception as exception:  # pylint: disable=broad-except
            print('error reading event: ' + exception)
            sys.exit(1)

    def handle(self):
        '''
        Method that should be overwritten to provide handler logic.
        '''
        print('ignoring event -- no handler defined')

    def filter(self):
        '''
        Filters exit the proccess if the event should not be handled.
        Filtering events is deprecated and will be removed in a future release.
        '''

        if self.deprecated_filtering_enabled():
            print('warning: event filtering in sensu-plugin is deprecated,' +
                  'see http://bit.ly/sensu-plugin')
            self.filter_disabled()
            self.filter_silenced()
            self.filter_dependencies()

            if self.deprecated_occurrence_filtering():
                print('warning: occurrence filtering in sensu-plugin is' +
                      'deprecated, see http://bit.ly/sensu-plugin')
                self.filter_repeated()

    def deprecated_filtering_enabled(self):
        '''
        Evaluates whether the event should be processed by any of the
        filter methods in this library. Defaults to true,
        i.e. deprecated filters are run by default.

        returns bool
        '''
        return self.event['check'].get('enable_deprecated_filtering', False)

    def deprecated_occurrence_filtering(self):
        '''
        Evaluates whether the event should be processed by the
        filter_repeated method. Defaults to true, i.e. filter_repeated
        will filter events by default.

        returns bool
        '''

        return self.event['check'].get(
            'enable_deprecated_occurrence_filtering', False)

    def bail(self, msg):
        '''
        Gracefully terminate with message
        '''
        client_name = self.event.get('client', 'error:no-client-name')
        check_name = self.event['client'].get('name', 'error:no-check-name')
        print('{}: {}/{}'.format(msg, client_name, check_name))
        sys.exit(0)

    def get_api_settings(self):
        '''
        Return a hash of API settings derived first from ENV['sensu_api_url']
        if set, then Sensu config `api` scope if configured, and finally
        falling back to to ipv4 localhost address on default API port.

        return dict
        '''

        sensu_api_url = os.environ.get('sensu_api_url')
        if sensu_api_url:
            uri = urlparse(sensu_api_url)
            self.api_settings = {
                'host': '{0}//{1}'.format(uri.scheme, uri.hostname),
                'port': uri.port,
                'user': uri.username,
                'password': uri.password
            }
        else:
            self.api_settings = self.settings.get('api', {})
            self.api_settings['host'] = self.api_settings.get(
                'host', '127.0.0.1')
            self.api_settings['port'] = self.api_settings.get(
                'port', 4567)

    # API requests
    def api_request(self, method, path):
        '''
        Query Sensu api for information.
        '''
        if not hasattr(self, 'api_settings'):
            ValueError('api.json settings not found')

        if method.lower() == 'get':
            _request = requests.get
        elif method.lower() == 'post':
            _request = requests.post

        domain = self.api_settings['host']
        uri = 'http://{}:{}{}'.format(domain, self.api_settings['port'], path)
        if self.api_settings['user'] and self.api_settings['password']:
            auth = (self.api_settings['user'], self.api_settings['password'])
        else:
            auth = ()
        req = _request(uri, auth=auth)
        return req

    def stash_exists(self, path):
        '''
        Query Sensu API for stash data.
        '''
        return self.api_request('get', '/stash' + path).status_code == 200

    def event_exists(self, client, check):
        '''
        Query Sensu API for event.
        '''
        return self.api_request(
            'get',
            'events/{}/{}'.format(client, check)
            ).status_code == 200

    # Filters
    def filter_disabled(self):
        '''
        Determine whether a check is disabled and shouldn't handle.
        '''
        if self.event['check']['alert'] is False:
            self.bail('alert disabled')

    def filter_silenced(self):
        '''
        Determine whether a check is silenced and shouldn't handle.
        '''
        stashes = [
            ('client', '/silence/{}'.format(self.event['client']['name'])),
            ('check', '/silence/{}/{}'.format(
                self.event['client']['name'],
                self.event['check']['name'])),
            ('check', '/silence/all/{}'.format(self.event['check']['name']))
        ]
        for scope, path in stashes:
            if self.stash_exists(path):
                self.bail(scope + ' alerts silenced')

    def filter_dependencies(self):
        '''
        Determine whether a check has dependencies.
        '''
        dependencies = self.event['check'].get('dependencies', None)
        if dependencies is None or not isinstance(dependencies, list):
            return
        for dependency in self.event['check']['dependencies']:
            if not str(dependency):
                continue
            dependency_split = tuple(dependency.split('/'))
            # If there's a dependency on a check from another client, then use
            # that client name, otherwise assume same client.
            if len(dependency_split) == 2:
                client, check = dependency_split
            else:
                client = self.event['client']['name']
                check = dependency_split[0]
            if self.event_exists(client, check):
                self.bail('check dependency event exists')

    def filter_repeated(self):
        '''
        Determine whether a check is repeating.
        '''
        defaults = {
            'occurrences': 1,
            'interval': 30,
            'refresh': 1800
        }

        # Override defaults with anything defined in the settings
        if isinstance(self.settings['sensu_plugin'], dict):
            defaults.update(self.settings['sensu_plugin'])

        occurrences = int(self.event['check'].get(
            'occurrences', defaults['occurrences']))
        interval = int(self.event['check'].get(
            'interval', defaults['interval']))
        refresh = int(self.event['check'].get(
            'refresh', defaults['refresh']))

        if self.event['occurrences'] < occurrences:
            self.bail('not enough occurrences')

        if (self.event['occurrences'] > occurrences and
                self.event['action'] == 'create'):
            return

        number = int(refresh / interval)
        if (number == 0 or
                (self.event['occurrences'] - occurrences) % number == 0):
            return

        self.bail('only handling every ' + str(number) + ' occurrences')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 - S. Zachariah Sprackett <zac@sprackett.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

from __future__ import print_function
import atexit
import sys
import argparse
import os
import traceback
from collections import namedtuple
from sensu_plugin.exithook import ExitHook

ExitCode = namedtuple('ExitCode', ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN'])


class SensuPlugin(object):
    def __init__(self):
        self.plugin_info = {
            'check_name': None,
            'message': None,
            'status': None
        }
        self._hook = ExitHook()
        self._hook.hook()

        self.exit_code = ExitCode(0, 1, 2, 3)
        for field in self.exit_code._fields:
            self.__make_dynamic(field)

        atexit.register(self.__exitfunction)

        self.parser = argparse.ArgumentParser()
        if hasattr(self, 'setup'):
            self.setup()
        (self.options, self.remain) = self.parser.parse_known_args()

        self.run()

    def output(self, args):
        print("SensuPlugin: %s" % ' '.join(str(a) for a in args))

    def __make_dynamic(self, method):

        def dynamic(*args):
            self.plugin_info['status'] = method
            if not args:
                args = None
            self.output(args)
            sys.exit(getattr(self.exit_code, method))

        method_lc = method.lower()
        dynamic.__doc__ = "%s method" % method_lc
        dynamic.__name__ = method_lc
        setattr(self, dynamic.__name__, dynamic)

    def run(self):
        self.warning("Not implemented! You should override SensuPlugin.run()")

    def __exitfunction(self):
        if self._hook.exit_code is None and self._hook.exception is None:
            print("Check did not exit! You should call an exit code method.")
            sys.stdout.flush()
            os._exit(1)
        elif self._hook.exception:
            print("Check failed to run: %s, %s" %
                  (sys.last_type, traceback.format_tb(sys.last_traceback)))
            sys.stdout.flush()
            os._exit(2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 - S. Zachariah Sprackett <zac@sprackett.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

import sys


class ExitHook(object):
    def __init__(self):
        self._orig_exit = None
        self.exit_code = None
        self.exception = None

    def hook(self):
        self._orig_exit = sys.exit
        sys.exit = self.exit
        sys.excepthook = self.exc_handler

    def exit(self, code=0):
        self.exit_code = code
        self._orig_exit(code)

    def exc_handler(self, _exc_type, exc, *_args):
        self.exception = exc



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 - S. Zachariah Sprackett <zac@sprackett.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

from __future__ import print_function
import json
import time
from sensu_plugin.plugin import SensuPlugin
from sensu_plugin.compat import compat_basestring


class SensuPluginMetricJSON(SensuPlugin):
    def output(self, args):
        obj = args[0]
        if isinstance(obj, (Exception, compat_basestring)):
            print(obj)
        elif isinstance(obj, (dict, list)):
            print(json.dumps(obj))


class SensuPluginMetricGraphite(SensuPlugin):
    def output(self, *args):
        if args[0] is None:
            print()
        elif isinstance(args[0], Exception) or args[1] is None:
            print(args[0])
        else:
            l_args = list(args)
            if len(l_args) < 3:
                l_args.append(None)
            if l_args[2] is None:
                l_args[2] = int(time.time())
            print("\t".join(str(s) for s in l_args[0:3]))


class SensuPluginMetricStatsd(SensuPlugin):
    def output(self, *args):
        if args[0] is None:
            print()
        elif isinstance(args[0], Exception) or args[1] is None:
            print(args[0])
        else:
            l_args = list(args)
            if len(l_args) < 3 or l_args[2] is None:
                stype = 'kv'
            else:
                stype = l_args[2]
            print("|".join([":".join(str(s) for s in l_args[0:2]), stype]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=undefined-variable
"""
Python 2/3 compatibility code.
"""

try:
    compat_basestring = basestring
except NameError:  # Python 3
    compat_basestring = (bytes, str)



import socket
import json


def push_event(sensu_client_host='127.0.0.1',
               sensu_client_port=3030,
               source=None,
               check_name=None,
               exit_code=None,
               message=None,
               **extra_vars):

    for param in [source, check_name, exit_code, message]:
        if param is None:
            raise ValueError
    message = {
        "source":  source,
        "name":    check_name,
        "status":  exit_code,
        "output":  message,
        }
    for key in extra_vars:
        message[key] = extra_vars[key]
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    except socket.error:
        sock.close()
        raise
    sock.connect((sensu_client_host, sensu_client_port))
    sock.send(json.dumps(message))
    if sock.recv(24) != "ok":
        sock.close()
        raise socket.error



from distutils.core import setup

setup(
    name='sensu_plugin',
    version='0.4.0',
    author='Sensu-Plugins and contributors',
    author_email='sensu-users@googlegroups.com',
    packages=['sensu_plugin', 'sensu_plugin.test'],
    scripts=[],
    url='https://github.com/sensu-plugins/sensu-plugin-python',
    license='LICENSE.txt',
    description='A framework for writing Python sensu plugins.',
    long_description="""
    """,
    install_requires=[
        'argparse'
    ],
    tests_require=[
        'pep8',
        'pylint'
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 - S. Zachariah Sprackett <zac@sprackett.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

from __future__ import print_function
from sensu_plugin.plugin import SensuPlugin


class SensuPluginCheck(SensuPlugin):
    def check_name(self, name=None):
        if name:
            self.plugin_info['check_name'] = name

        if self.plugin_info['check_name'] is not None:
            return self.plugin_info['check_name']

        return self.__class__.__name__

    def message(self, *m):
        self.plugin_info['message'] = m

    def output(self, args):
        msg = ''
        if args is None or (args[0] is None and len(args) == 1):
            args = self.plugin_info['message']

        if args is not None and not (args[0] is None and len(args) == 1):
            msg = ": {0}".format(' '.join(str(message) for message in args))

        print("{0} {1}{2}".format(self.check_name(),
                                  self.plugin_info['status'], msg))
